
import math, os
import numpy as np
import scipy.signal
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
from PIL import Image



def distanceAzimuthOnSphere(lon1, lat1, lon2, lat2):

	DegToRad = 3.14159265359/180.0
	phi1 = lat1*DegToRad; phi2 = lat2*DegToRad;
	theta1 = lon1*DegToRad; theta2=lon2*DegToRad;
	sind = math.sqrt((math.cos(theta1)*math.cos(phi1)-math.cos(theta2)*math.cos(phi2))**2 + \
	(math.sin(theta1)*math.cos(phi1)-math.sin(theta2)*math.cos(phi2))**2 + \
	(math.sin(phi1)-math.sin(phi2))**2) * \
	math.sqrt((math.cos(theta1)*math.cos(phi1)+math.cos(theta2)*math.cos(phi2))**2 + \
	(math.sin(theta1)*math.cos(phi1)+math.sin(theta2)*math.cos(phi2))**2 + \
	(math.sin(phi1)+math.sin(phi2))**2) * 0.5
	cosd = math.sin(phi1)*math.sin(phi2)+math.cos(phi1)*math.cos(phi2)*math.cos(theta1-theta2)
	dis = math.atan2(sind, cosd)/DegToRad
	az = math.atan2((math.cos(phi2)*math.sin(theta2-theta1))/sind, \
	(math.sin(phi2)-math.sin(phi1)*cosd)/math.cos(phi1)/sind)/DegToRad
	if(az < 0.0):
		az = az + 360.0
	baz = math.atan2(math.cos(phi1)*math.sin(theta2-theta1)/sind, \
	(math.sin(phi1)-math.sin(phi2)*cosd)/math.cos(phi2)/sind)/DegToRad
	baz = 360.0 - baz
	if(baz >= 360.0):
		baz = baz - 360.0
	return [dis, az, baz]



def generateHTMLViewer(AllParams, usgs, dart, ioc):

	os.chdir(AllParams.CachePath)

	epi_lon = float(usgs.AllEarthquakeInfo[usgs.EQID][2])
	epi_lat = float(usgs.AllEarthquakeInfo[usgs.EQID][1])
	epi_dep = float(usgs.AllEarthquakeInfo[usgs.EQID][3])

	x1 = epi_lon - 2.0; y1 = max(epi_lat-2.0, -89.0)
	x2 = epi_lon + 2.0; y2 = min(epi_lat+2.0,  89.0)
	if(x1 < 0.0):
		x1 += 360.0
	elif(x1 > 360.0):
		x1 -= 360.0
	if(x2 < 0.0):
		x2 += 360.0
	elif(x2 > 360.0):
		x2 -= 360.0
	if(x1 > x2):
		x1 -= 360.0
		StartEastWest = 2
	else:
		StartEastWest = 1
	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			sta  = datatype.AllStationsInfo[ista][0]
			lon  = datatype.AllStationsInfo[ista][1]
			lat  = datatype.AllStationsInfo[ista][2]
			if(StartEastWest == 1):
				if(lon < 0.0):
					lon += 360.0
				if(lon < x1):
					if(x1-lon < 360.0-(x2-lon)):
						x1 = lon
					else:
						x1 -= 360.0
						x2 = lon
						StartEastWest = 2
				elif(lon > x2):
					if(lon-x2 < 360.0-(lon-x1)):
						x2 = lon
					else:
						x1 = lon-360.0
						x2 = x2
						StartEastWest = 2
			if(StartEastWest == 2):
				if(lon > 180.0):
					lon -= 360.0
				if(lon < x1):
					x1 = lon
				elif(lon > x2):
					x2 = lon
			if(lat < y1):
				y1 = lat
			if(lat > y2):
				y2 = lat
	xlen = min(math.fabs(x2-x1), 360-math.fabs(x2-x1))
	ylen = y2-y1
	
	per = 0.1
	x1 = x1-0.1*xlen
	x2 = x2+0.1*xlen
	y1 = max(y1-0.1*ylen, -90)
	y2 = min(y2+0.1*ylen, 90)
	xlen = min(math.fabs(x2-x1), 360-math.fabs(x2-x1))
	ylen = y2-y1

	xmin = x1; ymin = y1; xmax = x2; ymax = y2

	fig = plt.figure(figsize=(15,10))
	m = Basemap(projection='merc', resolution='c', llcrnrlon=x1, llcrnrlat=y1, urcrnrlon=x2, urcrnrlat=y2,\
		lat_0 = epi_lat, lon_0 = epi_lon)
	m.etopo(scale=1.0, alpha=0.8)
	m.drawcoastlines()

	i = max(1,int((x2-x1)/5))
	if(i == 3):
		i = 2
	elif(i >= 4 and i <= 7):
		i = 5
	elif(i >= 8):
		i = 10
	meridians = np.arange(int(x1),int(x2),i)
	m.drawmeridians(meridians, color=[0.3,0.3,0.3], linewidth=0.5, labels=[0,0,1,1], dashes=[1,0])

	i = max(1,int((y2-y1)/4))
	if(i == 3):
		i = 2
	elif(i >= 4 and i <= 7):
		i = 5
	elif(i >= 8):
		i = 10
	parallels = np.arange(int(y1),int(y2),i)
	m.drawparallels(parallels, color=[0.3,0.3,0.3], linewidth=0.5, labels=[1,1,0,0], dashes=[1,0])

	if(StartEastWest == 1 and epi_lon < 0.0):
		epi_lon += 360.0
	if(StartEastWest == 2 and epi_lon > 180.0):
		epi_lon -= 360.0
	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			lon  = datatype.AllStationsInfo[ista][1]
			lat  = datatype.AllStationsInfo[ista][2]
			if(StartEastWest == 1 and datatype.AllStationsInfo[ista][1] < 0.0):
				datatype.AllStationsInfo[ista][1] += 360.0
			if(StartEastWest == 2 and datatype.AllStationsInfo[ista][1] > 180.0):
				datatype.AllStationsInfo[ista][1] -= 360.0
	AllLon = []; AllLat = []
	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			lon  = datatype.AllStationsInfo[ista][1]
			lat  = datatype.AllStationsInfo[ista][2]
			for j in range(0, len(AllLon)):
				per = 0.02
				if(abs(lon-AllLon[j])<=per*xlen and abs(lat-AllLat[j])<=per*ylen):
					lon = min(lon+per*xlen,xmax); datatype.AllStationsInfo[ista][1] = lon
					lat = min(lat+per*ylen,ymax); datatype.AllStationsInfo[ista][2] = lat
			AllLon.append(lon); AllLat.append(lat)

	[x1, y1] = m(epi_lon, epi_lat)
	plt.plot(x1, y1, '*r', markersize=20)
	#x2, y2 = m(epi_lon+0.01*xlen, epi_lat)
	#plt.text(x2, y2, 'dep='+str('%0.1f'%epi_dep)+'km', fontsize=14,color = 'r')

	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			sta  = datatype.AllStationsInfo[ista][0]
			lon  = datatype.AllStationsInfo[ista][1]
			lat  = datatype.AllStationsInfo[ista][2]
			[x1, y1] = m(lon, lat)
			[x2, y2] = m(lon+0.01*xlen, lat)
			if(datatype == dart):
				plt.plot(x1, y1, '^m', markersize=9)
				plt.text(x2, y2, sta, fontsize=12, color='magenta')
			elif(datatype == ioc):
				plt.plot(x1, y1, 'ok', markersize=9)
				plt.text(x2, y2, sta, fontsize=12, color='black')

	plt.savefig('map.png',bbox_inches='tight', pad_inches=0.0)
	w0, h0 = Image.open('map.png').size

	xmin_meter, ymin_meter = m(xmin, ymin)
	xmax_meter, ymax_meter = m(xmax, ymax)

	s  = ''
	s += '<!DOCTYPE html>\n<html>\n<head>\n'
	s += ' '*4+'<style>\n'
	s += ' '*8+'.ImageContainer\n'
	s += ' '*8+'{\n'
	s += ' '*12+'position: relative; top: 0px; left: 0px;\n'
	s += ' '*8+'}\n\n'
	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			sta  = datatype.AllStationsInfo[ista][0]
			lon  = datatype.AllStationsInfo[ista][1]
			lat  = datatype.AllStationsInfo[ista][2]
			x1, y1 = m(lon, lat-0.02*ylen)
			xpos = int((x1-xmin_meter)/(xmax_meter-xmin_meter)*w0)
			ypos = int((ymax_meter-y1)/(ymax_meter-ymin_meter)*h0)
			s += ' '*8+'.StationArea'+str(ista)+'\n'
			s += ' '*8+'{\n'
			s += ' '*12+'height: 20px; width: 50px; position: absolute; '
			s += 'top: '+str(ypos)+'px; left: '+str(xpos)+'px; '
			if(datatype == dart):
				s += 'background: magenta;\n'
			elif(datatype == ioc):
				s += 'background: white;\n'
			s += ' '*8+'}\n'
			s += ' '*8+'.hiddenimg'+str(ista)+'\n'
			s += ' '*8+'{\n'
			s += ' '*12+'display: none; position: absolute; z-index:1000; '
			s += 'top: '+str(-ypos+5)+'px; left: '+str(-xpos+5)+'px;\n'
			s += ' '*8+'}\n'
			s += ' '*8+'.StationArea'+str(ista)+':hover .hiddenimg'+str(ista)+'\n'
			s += ' '*8+'{\n'
			s += ' '*12+'display: block;\n'
			s += ' '*8+'}'
	s += '\n'+' '*4+'</style>\n'
	s += ' '*4+'<title>Tsunami Data</title>\n'
	s += '</head>'

	s += '\n<body>\n\n'
	s1  = usgs.AllEarthquakeInfo[usgs.EQID][0]
	s1 += '&nbsp;'*6+'lon='+str('%0.3f'%(float(usgs.AllEarthquakeInfo[usgs.EQID][2])))
	s1 += ',&nbsp;lat='+str('%0.3f'%(float(usgs.AllEarthquakeInfo[usgs.EQID][1])))
	s1 += ',&nbsp;dep='+str('%0.1f'%(float(usgs.AllEarthquakeInfo[usgs.EQID][3])))+'km'
	s1 += '&nbsp;'*6+usgs.AllEarthquakeInfo[usgs.EQID][5]+str('%0.1f'%(float(usgs.AllEarthquakeInfo[usgs.EQID][4])))
	s1 += '&nbsp;'*6+usgs.AllEarthquakeInfo[usgs.EQID][13].replace('"','')
	s += ' '*4+'<h3>'+s1+'</h3>\n\n'

	s += ' '*4+'<div class="ImageContainer">\n\n'
	s += ' '*8+'<img src="./cache/map.png" >\n\n'

	for datatype in [dart, ioc]:
		for i in range(0, len(datatype.StationsDataDownloaded)):
			ista = datatype.StationsDataDownloaded[i]
			sta  = datatype.AllStationsInfo[ista][0]
			w, h = Image.open(sta+'.png').size
			s += ' '*8+'<div class="StationArea'+str(ista)+'">\n'
			s += ' '*12+sta+'\n'
			s += ' '*12+'<span class="hiddenimg'+str(ista)+'"> <img src="./cache/'+sta+'.png" alt="" '
			s += 'width="'+str(min(1200, w))+'"> </span>\n'
			s += ' '*8+'</div>\n'
	s += '\n'+' '*4+'</div\n\n'
	s += '</body>\n</html>'

	os.chdir(AllParams.MainPath)

	f = open('MainMap.html', 'w')
	f.write(s)
	f.close()



def makeStationPlot(fn, DataLengthToShow):

	dat = np.loadtxt(fn)
	t = dat[:,0]
	h = dat[:,1]

	fndat = fn[0:len(fn)-4]+'_filt.dat'
	dat = np.loadtxt(fndat)
	t_res_filt = dat[:,0]
	h_res_filt = dat[:,1]

	h_rem_filt = np.interp(t, t_res_filt, h_res_filt)
	h_rem_filt = h - h_rem_filt

	fndat = fn[0:len(fn)-4]+'_poly.dat'
	dat = np.loadtxt(fndat)
	t_res_poly = dat[:,0]
	h_res_poly = dat[:,1]

	h_rem_poly = h - h_res_poly

	plt.rcParams.update({'font.size': 10})
	fig, ax = plt.subplots(nrows=1, ncols=2, figsize=(12,4))

	ax[0].plot(t/60, h, color='black', linewidth=2, label='raw')
	ax[0].plot(t/60, h_rem_filt, color='blue', linewidth=1, label='filter')
	ax[0].plot(t/60, h_rem_poly, color='red', linewidth=1, label='poly')
	ax[0].set_xlabel('Time (min)')
	ax[0].set_ylabel('Elevation (m)')
	ax[0].set_title(os.path.basename(fn)+' : raw and fit')
	ax[0].legend(loc='best')

	ax[1].plot(t_res_filt/60, h_res_filt, color='blue', linewidth=1, label='filter')
	ax[1].plot(t_res_poly/60, h_res_poly, color='red', linewidth=1, label='poly')
	ax[1].set_xlabel('Time (min)')
	plt.xlim(0, DataLengthToShow)
	ax[1].set_title(os.path.basename(fn)[0:-4]+'.dat')
	ax[1].legend(loc='best')

	plt.tight_layout()
	plt.savefig(fn[0:-4]+'.png',dpi=600,bbox_inches='tight')



def removeStationTidesFilter(fn, maxperiod):

	dat = np.loadtxt(fn)
	t = dat[:,0]
	h = dat[:,1]

	dtmax = -1.0; dtmin = 100000.0; dt = 0.0
	for i in range(1, np.size(t)):
		dt = math.fabs(t[i]-t[i-1])
		if(dt > dtmax):
			dtmax = dt
		if(dt < dtmin):
			dtmin = dt
	if(dtmax == dtmin):
		dt = dtmax
	else:
		dt = 60.0
		tnew = np.arange(t[0], t[np.size(t)-1], dt)
		hnew = np.interp(tnew, t, h)
		t = tnew
		h = hnew
	w = dt*2.0/(maxperiod*60.0)
	n0 = 7
	b, a = scipy.signal.butter(n0, w, 'high')
	hout = scipy.signal.filtfilt(b, a, h)

	fnnew = fn[0:len(fn)-4]+'_filt.dat'
	np.savetxt(fnnew, np.transpose([t,hout]), fmt='%0.1f %1.4e')



def removeStationTidesPolynomial(fn, PolyOrder):

	dat = np.loadtxt(fn)
	t = dat[:,0]
	h = dat[:,1]

	p = np.polyfit(t, h, PolyOrder)
	hpoly = np.polyval(p, t)
	hout = h-hpoly

	fnnew = fn[0:len(fn)-4]+'_poly.dat'
	np.savetxt(fnnew, np.transpose([t,hout]), fmt='%0.1f %1.4e')
