
import re, os, math, datetime, sys, json, shutil
from FunctionsLib import distanceAzimuthOnSphere, makeStationPlot, removeStationTidesFilter, removeStationTidesPolynomial



class TsunamiData():


	CachePath = ''
	DataType = ''; # DART; IOC
	AutoChooseStations = -1;
	StationsToDownloadDataMaxNum = 60



	def __init__(self, CachePath, DataType):


		self.AllStationsInfo = []
		self.AllStationsNames = []
		self.AllStationsNamesLower = []

		self.AutoChooseStations = -1;
		self.StationsToDownloadData = []
		self.StationsDataDownloaded = []

		self.CachePath = CachePath
		if(os.path.isdir(CachePath) == False):
			os.mkdir(CachePath)
		if(DataType.lower() == 'dart'):
			self.DataType = 'dart'
			self.urlStationsList = 'http://www.ndbc.noaa.gov/dart.shtml'
			self.urlStationInfoPrefix = 'http://www.ndbc.noaa.gov/station_page.php'
			self.urlStationDataPrefix = 'http://www.ndbc.noaa.gov/station_page.php'
			self.DataPageHeader = '#YY  MM DD hh mm ss T   HEIGHT'
			self.DataPageTail   = '</textarea></label></pre>'
		elif(DataType.lower() == 'ioc'):
			self.DataType = 'ioc'
			self.urlStationsList = 'http://www.ioc-sealevelmonitoring.org/ssc/service.php?format=json'
			self.urlStationInfoPrefix = ''
			self.urlStationDataPrefix = 'http://www.ioc-sealevelmonitoring.org/bgraph.php'
			self.DataPageHeader = ''
			self.DataPageTail   = ''
		else:
			print('\nUnknown TsunamiData Option (dart or ioc): ', DataType)
			exit(0)
		self.StationsInfoFileName = 'All'+self.DataType.upper()+'Stations.csv'

		self.readStationsInfo()



	def chooseStations(self, AllParams, usgs):

		self.StationsToDownloadData = []
		if(self.DataType.lower() == 'dart'):
			self.AutoChooseStations = AllParams.AutoChooseDARTStations
			SelectedStations = AllParams.DARTStationsList
		elif(self.DataType.lower() == 'ioc'):
			self.AutoChooseStations = AllParams.AutoChooseIOCStations
			SelectedStations = AllParams.IOCStationsList

		lon = float(usgs.AllEarthquakeInfo[usgs.EQID][2])
		lat = float(usgs.AllEarthquakeInfo[usgs.EQID][1])
		stations = self.AllStationsNames

		if(self.AutoChooseStations <= 0):
			for ista in range(0, len(SelectedStations)):
				sta = SelectedStations[ista].lower()
				if(sta in self.AllStationsNamesLower):
					self.StationsToDownloadData.append(self.AllStationsNamesLower.index(sta))
				else:
					self.StationsToDownloadData.append(SelectedStations[ista])
		else:
			dis = []; sta = []
			for ista in range(0, len(self.AllStationsNames)):
				lon_sta = self.AllStationsInfo[ista][1]
				lat_sta = self.AllStationsInfo[ista][2]
				[dist, az, baz] = distanceAzimuthOnSphere(lon,lat,lon_sta,lat_sta)
				dis.append(dist)
				sta.append(ista)
			for i in range(0, len(self.AllStationsNames)-1):
				for j in range(i+1, len(self.AllStationsNames)):
					if(dis[i] > dis[j]):
						tmp = dis[i]; dis[i] = dis[j]; dis[j] = tmp
						tmp = sta[i]; sta[i] = sta[j]; sta[j] = tmp
			self.StationsToDownloadData = [x for x in sta[0:min(len(sta),self.StationsToDownloadDataMaxNum)]]

	

	def createGMTFile(self):

		s1 = ''; s2 = ''
		for ista in range(0, len(self.AllStationsInfo)):
			s1 += str('%0.3f'%self.AllStationsInfo[ista][1])+'\t'+str('%0.3f'%self.AllStationsInfo[ista][2])+'\t'\
				+self.AllStationsInfo[ista][0]+'\n'
			s2 += str('%0.3f'%self.AllStationsInfo[ista][1])+'\t'+str('%0.3f'%self.AllStationsInfo[ista][2])+'\t'\
				'20\t0\t0\tTC\t'+self.AllStationsInfo[ista][0]+'\n'
		f1 = open(os.path.join(self.CachePath,'StationsLocationAll'+self.DataType+'.gmttxt'),'w')
		f1.write(s1.strip())
		f1.close()
		f2 = open(os.path.join(self.CachePath,'StationsNamesAll'+self.DataType+'.gmttxt'),'w')
		f2.write(s2.strip())
		f2.close()


		fn1 = os.path.join(self.CachePath,'StationsLocation'+self.DataType+'.gmttxt')
		fn2 = os.path.join(self.CachePath,'StationsNames'+self.DataType+'.gmttxt')
		fn3 = os.path.join(self.CachePath,'Stations'+self.DataType+'.ctl')

		if(len(self.StationsDataDownloaded) == 0):
			if(os.path.isfile(fn1) == True):
				os.remove(fn1)
			if(os.path.isfile(fn2) == True):
				os.remove(fn2)
			if(os.path.isfile(fn3) == True):
				os.remove(fn3)
			return

		s1 = ''; s2 = ''; s3 = ''
		for i in range(0, len(self.StationsDataDownloaded)):
			ista = self.StationsDataDownloaded[i]
			s1 += str('%0.3f'%self.AllStationsInfo[ista][1])+'\t'+str('%0.3f'%self.AllStationsInfo[ista][2])+'\t'\
				+self.AllStationsInfo[ista][0]+'\n'
			s2 += str('%0.3f'%self.AllStationsInfo[ista][1])+'\t'+str('%0.3f'%self.AllStationsInfo[ista][2])+'\t'\
				'20\t0\t0\tTC\t'+self.AllStationsInfo[ista][0]+'\n'
			s3 += str('%0.3f'%self.AllStationsInfo[ista][1])+' '+str('%0.3f'%self.AllStationsInfo[ista][2])+' '\
				+self.AllStationsInfo[ista][0]+'\n'

		f1 = open(fn1, 'w'); f1.write(s1.strip()); f1.close()
		f2 = open(fn2, 'w'); f2.write(s2.strip()); f2.close()
		f3 = open(fn3, 'w'); f3.write(s3.strip()); f3.close()
	


	def downloadData(self, AllParams, usgs, wpd):

		if(self.DataType == 'dart'):
			self.downloadDataDART(AllParams, usgs, wpd)
		elif(self.DataType == 'ioc'):
			self.downloadDataIOC(AllParams, usgs, wpd)



	def downloadDataDART(self, AllParams, usgs, wpd):

		t0 = datetime.datetime.strptime(usgs.AllEarthquakeInfo[usgs.EQID][0], '%Y-%m-%d %H:%M:%S')
		ts = t0 + datetime.timedelta(days=-AllParams.nDataLengthDay)
		te = t0 + datetime.timedelta(days=+AllParams.nDataLengthDay+1)
		ts = datetime.datetime.strftime(ts, '%Y-%m-%d %H:%M:%S').split(' ')[0].split('-')
		te = datetime.datetime.strftime(te, '%Y-%m-%d %H:%M:%S').split(' ')[0].split('-')

		urlPrefix  = self.urlStationDataPrefix+'?type=0'
		urlPrefix += '&startyear='+ts[0]+'&startmonth='+ts[1]+'&startday='+ts[2]
		urlPrefix += '&endyear='+te[0]+'&endmonth='+te[1]+'&endday='+te[2]
		urlPrefix += '&submit=Submit&station='

		self.StationsDataDownloaded = []
		for i in range(0, len(self.StationsToDownloadData)):

			ista = self.StationsToDownloadData[i]
			if(isinstance(ista,str) == True):
				sys.stdout.write((ista+' (NO_STATION)').ljust(15,' ')); sys.stdout.flush()
				continue
			station = self.AllStationsNamesLower[ista]
			if(i%5 == 0):
				print('\n'); sys.stdout.write(' '*4); sys.stdout.flush()
			sys.stdout.write(station.ljust(5,' ')); sys.stdout.flush()

			url = urlPrefix + station
			the_page = wpd.downloadWebpage(url)
			if(the_page.find(self.DataPageHeader) == -1):
				sys.stdout.write(' (NO_DATA)'.ljust(15,' ')); sys.stdout.flush()
				continue
			s = self.DataPageHeader +  the_page.split(self.DataPageHeader)[1].split(self.DataPageTail)[0]
			l = s.strip().split('\n')

			t = []; d = []
			for s in l:
				if s.startswith('#'):
					continue
				ll = re.split('\s+', s)
				t1 = datetime.datetime.strptime(' '.join(ll[0:6]), '%Y %m %d %H %M %S')
				t1 = (t1-t0).total_seconds()
				d1 = float(ll[7])
				if(math.fabs(d1) > 9000.0):
					continue
				if(len(t) > 0 and t1 == t[-1]):
					continue
				t.insert(0, t1); d.insert(0, d1)
			
			if(len(t) == 0):
				sys.stdout.write(' (NO_DATA)'.ljust(15,' ')); sys.stdout.flush()
				continue

			fn = os.path.join(self.CachePath, station+'.txt')
			f = open(fn, 'w')
			for i in range(0, len(t)):
				f.write(str(t[i]).ljust(20,' ')+str(d[i]))
				if(i != len(t)-1):
					f.write('\n')
			f.close()
			sys.stdout.write(' '*15); sys.stdout.flush()
			self.StationsDataDownloaded.append(ista)

			if(self.AutoChooseStations > 0 and len(self.StationsDataDownloaded) == self.AutoChooseStations):
				break

		print('')
		self.createGMTFile()



	def downloadDataIOC(self, AllParams, usgs, wpd):

		t0 = datetime.datetime.strptime(usgs.AllEarthquakeInfo[usgs.EQID][0], '%Y-%m-%d %H:%M:%S')
		ts = t0 + datetime.timedelta(days=-AllParams.nDataLengthDay)
		te = t0 + datetime.timedelta(days=AllParams.nDataLengthDay+2)
		tlen = (te-ts).total_seconds()/24/3600.0
		tlen = min(tlen,30)
		ts = datetime.datetime.strftime(ts, '%Y-%m-%d %H:%M:%S').split(' ')[0].split('-')
		te = datetime.datetime.strftime(te, '%Y-%m-%d %H:%M:%S').split(' ')[0].split('-')

		urlPrefix  = self.urlStationDataPrefix
		urlPrefix += '?output=tab&period='+str('%0.4f'%tlen)
		urlPrefix += '&endtime='+'-'.join(te)
		urlPrefix += '&code='

		SensorOrder = ['prs','rad','aqu']
		SensorDataExist = [-1]*len(SensorOrder)

		self.StationsDataDownloaded = []
		for i in range(0, len(self.StationsToDownloadData)):

			ista = self.StationsToDownloadData[i]
			if(isinstance(ista,str) == True):
				sys.stdout.write((ista+' (NO_STATION)').ljust(15,' ')); sys.stdout.flush()
				continue
			station = self.AllStationsNamesLower[ista]
			if(i%5 == 0):
				print('\n'); sys.stdout.write(' '*4); sys.stdout.flush()
			sys.stdout.write(station.ljust(5,' ')); sys.stdout.flush()

			url = urlPrefix + station
			the_page = wpd.downloadWebpage(url)
			l = the_page.strip().split('<tr>')
			
			if(len(l) <= 3):
				sys.stdout.write(' (NO_DATA)'.ljust(15,' ')); sys.stdout.flush()
				continue

			SensorDataExist = [-1]*len(SensorOrder)
			ll = l[1].split('</td>')
			for i in range(0, len(SensorOrder)):
				for j in range(0, len(ll)):
					if(ll[j].find(SensorOrder[i]+'(m)') != -1):
						SensorDataExist[i] = j
						break
			t = []; d = []
			for i in range(0, len(SensorOrder)):
				t_this = []; d_this = []
				if(SensorDataExist[i] > 0):
					for idat in range(2, len(l)):
						ll = [x.replace('<td>','') for x in l[idat].split('</td>')]
						t1 = datetime.datetime.strptime(ll[0], '%Y-%m-%d %H:%M:%S')
						t1 = (t1-t0).total_seconds()
						try:
							d1 = float(ll[SensorDataExist[i]])
							t_this.append(t1); d_this.append(d1)
						except ValueError as e:
							pass
				if(SensorDataExist[i] > 0 and len(t_this) <= 5):
					SensorDataExist[i] = -1
				t.append(t_this); d.append(d_this)

			fn_sensor = []
			for i in range(0, len(SensorOrder)):
				fn1 = os.path.join(self.CachePath,station+'_'+SensorOrder[i]+'.txt')
				fn_sensor.append(fn1)
				if(SensorDataExist[i] > 0):
					f = open(fn1, 'w')
					for j in range(0, len(t[i])):
						f.write(str(t[i][j]).ljust(20,' ')+str(d[i][j]))
						if(i != len(t[i])-1):
							f.write('\n')
					f.close()

			icopy = -1; t_num = -1
			for i in range(0, len(SensorOrder)):
				if(SensorDataExist[i] > 0 and len(t[i]) > t_num):
					icopy = i; t_num = len(t[i])
			if(icopy == -1):
				sys.stdout.write(' (NO_DATA)'.ljust(15,' ')); sys.stdout.flush()
				continue
			
			fn = os.path.join(self.CachePath,station+'.txt')
			shutil.copy(fn_sensor[icopy], fn)
			
			sys.stdout.write(('('+SensorOrder[icopy]+')').ljust(15,' ')); sys.stdout.flush()
			self.StationsDataDownloaded.append(ista)

			if(self.AutoChooseStations > 0 and len(self.StationsDataDownloaded) == self.AutoChooseStations):
				break

		print('')
		self.createGMTFile()



	def makePlotsAllStations(self, AllParams):

		for i in range(0, len(self.StationsDataDownloaded)):

			ista = self.StationsDataDownloaded[i]
			sta  = self.AllStationsNamesLower[ista]

			fn = os.path.join(self.CachePath, sta+'.txt')
			makeStationPlot(fn, AllParams.TsunamiDataLength)



	def readStationsInfo(self):
		
		if(os.path.isfile(self.StationsInfoFileName) == True):
			f = open(self.StationsInfoFileName, 'r')
			s = f.read().strip()
			f.close()
			self.AllStationsInfo = []
			sl = s.splitlines()
			for i in range(1,len(sl)):
				x = sl[i]
				l = x.strip().split(',')
				self.AllStationsInfo.append([l[0].strip(), float(l[1]), float(l[2]), float(l[3])])
				self.AllStationsNames.append(l[0].strip())
				self.AllStationsNamesLower.append(l[0].strip().lower())
	


	def removeTidesAllStations(self, AllParams):

		for i in range(0, len(self.StationsDataDownloaded)):

			ista = self.StationsDataDownloaded[i]
			sta  = self.AllStationsNamesLower[ista]

			fn = os.path.join(self.CachePath, sta+'.txt')
			removeStationTidesFilter(fn, AllParams.FilterMaxPeriod)
			removeStationTidesPolynomial(fn, AllParams.PolynomialOrder)
			if(AllParams.FilterOrPolynomial == 1):
				shutil.copy(fn[0:-4]+'_filt.dat',fn[0:-4]+'.dat')
			elif(AllParams.FilterOrPolynomial == 2):
				shutil.copy(fn[0:-4]+'_poly.dat',fn[0:-4]+'.dat')



	def saveStationsInfo(self):

		s = ''
		for ista in range(0, len(self.AllStationsInfo)):
			s += self.AllStationsInfo[ista][0]+','+str('%0.3f'%self.AllStationsInfo[ista][1])+','\
			    +str('%0.3f'%self.AllStationsInfo[ista][2])+','+str('%0.1f'%self.AllStationsInfo[ista][3])+'\n'
		f = open(self.StationsInfoFileName, 'w')
		f.write('Station,Longitude,Latitude,WaterDepth\n')
		f.write(s.strip())
		f.close()



	def updateStations(self, wpd):

		if(self.DataType == 'dart'):
			self.updateStationsDART(wpd)
		elif(self.DataType == 'ioc'):
			self.updateStationsIOC(wpd)



	def updateStationsDART(self, wpd):

		the_page = wpd.downloadWebpage(self.urlStationsList)
		Temp = the_page.split('dartstns = [')[1].split('];',1)[0].replace("'","")
		stations = [x.strip().lower() for x in Temp.split(',')]

		if(len(self.AllStationsInfo) > 0):
			stationsOld = []
			for ista in range(0, len(self.AllStationsInfo)):
				stationsOld.append(self.AllStationsInfo[ista][0])
			if(set(stations) == set(stationsOld)):
				print('\n    No need to refresh DART stations!')
				self.createGMTFile()
				return

		print('')
		longitude = []; latitude=[]; waterdepth=[]
		for station in stations:
			print('updating station ',station,' ...')
			the_page = wpd.downloadWebpage(self.urlStationInfoPrefix+'?station='+station)
			Temp = the_page.split(station+' (',1)[1].split(')',1)[0]
			try:
				Temp2 = the_page.split('Water depth:</b> ',1)[1].split(' m')[0]
			except IndexError as e:
				Temp2 = '-1'
				
			if re.match(r'(\d{1,3}.\d{3})N\s(\d{1,3}.\d{3})E',Temp):
				m = re.match(r'(\d{1,3}.\d{3})N\s(\d{1,3}.\d{3})E',Temp)
				latitude.append(m.group(1))
				longitude.append(m.group(2))
				waterdepth.append(Temp2)
				continue
			if re.match(r'(\d{1,3}.\d{3})S\s(\d{1,3}.\d{3})E',Temp):
				m = re.match(r'(\d{1,3}.\d{3})S\s(\d{1,3}.\d{3})E',Temp)
				latitude.append('-'+m.group(1))
				longitude.append(m.group(2))
				waterdepth.append(Temp2)
				continue
			if re.match(r'(\d{1,3}.\d{3})N\s(\d{1,3}.\d{3})W',Temp):
				m = re.match(r'(\d{1,3}.\d{3})N\s(\d{1,3}.\d{3})W',Temp)
				latitude.append(m.group(1))
				longitude.append('-'+m.group(2))
				waterdepth.append(Temp2)
				continue
			if re.match(r'(\d{1,3}.\d{3})S\s(\d{1,3}.\d{3})W',Temp):
				m = re.match(r'(\d{1,3}.\d{3})S\s(\d{1,3}.\d{3})W',Temp)
				latitude.append('-'+m.group(1))
				longitude.append('-'+m.group(2))
				waterdepth.append(Temp2)
				continue
		longitude = [float(x) for x in longitude]
		latitude = [float(x) for x in latitude]
		waterdepth = [float(x) for x in waterdepth]

		self.AllStationsInfo = []
		for ista in range(0, len(stations)):
			self.AllStationsInfo.append([stations[ista], longitude[ista], latitude[ista], waterdepth[ista]])

		self.saveStationsInfo()
		self.readStationsInfo()
		self.createGMTFile()
		print('\n    DART Station Update Complete!')



	def updateStationsIOC(self, wpd):

		the_page = wpd.downloadWebpage(self.urlStationsList)
		l = json.loads(the_page)
		stations = []; longitude = []; latitude = []
		for i in range(0, len(l)):
			stations.append(l[i]['ssc_id'][4:].lower())
			longitude.append(float(l[i]['geo:lon']))
			latitude.append(float(l[i]['geo:lat']))

		if(len(self.AllStationsInfo) > 0):
			stationsOld = []
			for ista in range(0, len(self.AllStationsInfo)):
				stationsOld.append(self.AllStationsInfo[ista][0])
			if(set(stations) == set(stationsOld)):
				print('\n    No need to refresh IOC stations!')
				self.createGMTFile()
				return

		self.AllStationsInfo = []
		for ista in range(0, len(stations)):
			self.AllStationsInfo.append([stations[ista], longitude[ista], latitude[ista], 0.0])
		self.saveStationsInfo()
		self.readStationsInfo()
		self.createGMTFile()
		print('\n    IOC Station Update Complete!')
