
import os, sys, datetime



class USGSEarthquakeInfo():

	CachePath = ''
	USGSEarthquakeCatalogFileName = ''
	
	urlUSGSPrefix = 'https://earthquake.usgs.gov/fdsnws/event/1/query.csv?'
	# time,latitude,longitude,depth,mag,magType,nst,gap,dmin,rms
	# net,id,updated,place,type,horizontalError,depthError,magError,magNst,status
	# locationSource,magSource
	AllEarthquakeInfoTitle = ''
	AllEarthquakeInfo = None

	EQID = -1



	def __init__(self, CachePath):

		if(os.path.isdir(CachePath) == False):
			os.mkdir(CachePath)
		self.USGSEarthquakeCatalogFileName = os.path.join(CachePath, 'earthquake_usgs.csv')
		self.readUSGSEarthquakes()



	def getUSGSEarthquakes(self, AllParams, wpd):

		url  = self.urlUSGSPrefix
		url += 'starttime='+AllParams.StartTime
		url += '&endtime='+AllParams.EndTime
		url += '&minmagnitude='+str(AllParams.MinMagnitude)
		url += '&maxdepth='+str(AllParams.MaxDepth)

		the_page = wpd.downloadWebpage(url)
		l = the_page.splitlines()
		self.AllEarthquakeInfoTitle = l[0]

		self.AllEarthquakeInfo = []
		for i in range(1, len(l)):
			x = l[i]
			ll = x.strip().split(',')
			ll[0] = ll[0].replace('T',' ').split('.')[0]
			self.AllEarthquakeInfo.append(ll)
		self.saveUSGSEarthquakes()



	def printUSGSEarthquakes(self):

		aei = self.AllEarthquakeInfo
		print('\n    id'+' '*4+'Time'+' '*19+'Mag'+' '*4+'Depth(km)'+' '*5+'Place')
		print(' '*4+'-'*80)
		for ieq in range(0, len(aei)):
			s  = ' '*4+str(ieq+1).rjust(2,' ')
			s += ' '*4+aei[ieq][0]
			s += ' '*4+str('%0.1f'%(float(aei[ieq][4])))
			s += ' '*3+str('%0.1f'%(float(aei[ieq][3]))).rjust(5,' ')
			s += ' '*10+aei[ieq][13].replace('"','')
			print(s)
	
	

	def readUSGSEarthquakes(self):

		if(os.path.isfile(self.USGSEarthquakeCatalogFileName) == True):
			f = open(self.USGSEarthquakeCatalogFileName, 'r')
			s = f.read()
			f.close()
			l = s.strip().splitlines()

			self.AllEarthquakeInfoTitle = l[0]

			self.AllEarthquakeInfo = []
			for i in range(1, len(l)):
				self.AllEarthquakeInfo.append(l[i].split(','))



	def saveUSGSEarthquakes(self):

		s = self.AllEarthquakeInfoTitle+'\n'
		for ieq in range(0, len(self.AllEarthquakeInfo)):
			s += ','.join(self.AllEarthquakeInfo[ieq])+'\n'

		f = open(self.USGSEarthquakeCatalogFileName, 'w')
		f.write(s.strip())
		f.close()



	def selectUSGSEarthquake(self):

		while True:
			command = input('\n--> Choose an earthquake ID to continue (0: manual input; 1: default; q: abort) ... ')
			if(command == 'q' or command == 'Q'):
				self.EQID = -1
				break
			if(command == ''):
				self.EQID = 0
				break
			if(command == '0'):
				print('\n--> Manually input source information (separated by comma, q to abort) ...\n')
				print(' '*4+'Year, Month, Day, Hour, Minute, Second, Latitude, Longigude, Description')
				while True:
					s0 = input(' '*4)
					if(s0.strip() == 'q' or s0.strip() == 'Q'):
						self.EQID = -1
						break
					l = s0.strip().split(',')
					if(len(l) != 8 and len(l) != 9):
						sys.stdout.write("\033[F") # back to previous line
						sys.stdout.write("\033[K") # clear previous line
						print(' '*4+s0+' '*2+'(Wrong number of commas)')
						continue
					try:
						t0 = datetime.datetime.strptime(' '.join([x.strip() for x in l[0:6]]), '%Y %m %d %H %M %S')
					except:
						sys.stdout.write("\033[F") # back to previous line
						sys.stdout.write("\033[K") # clear previous line
						print(' '*4+s0+' '*2+'(Cannot convert time)')
						continue
					try:
						lat = float(l[6])
						lon = float(l[7])
					except:
						sys.stdout.write("\033[F") # back to previous line
						sys.stdout.write("\033[K") # clear previous line
						print(' '*4+s0+' '*2+'(Cannot latitude and longitude)')
						continue
					thisEQ = ['']*22
					thisEQ[0] = datetime.datetime.strftime(t0,'%Y-%m-%d %H:%M:%S')
					thisEQ[1] = l[6]
					thisEQ[2] = l[7]
					thisEQ[3] = '0.0'
					thisEQ[4] = '0.0'
					if(len(l) == 9):
						thisEQ[13] = l[8]
					else:
						thisEQ[13] = 'None'
					self.AllEarthquakeInfo = []
					self.AllEarthquakeInfo.append(thisEQ)
					self.EQID = 0
					break
				break

			try:
				n = int(command)
				if(n >= 1 and n <= len(self.AllEarthquakeInfo)):
					self.EQID = n-1
					break
			except:
				pass
